@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -55,8 +54,8 @@ struct _GthEditCommentPagePrivate {
 
 
 void
-gth_edit_comment_page_real_set_file (GthEditMetadataPage *base,
-		 		     GthFileData         *file_data)
+gth_edit_comment_page_real_set_file_list (GthEditMetadataPage *base,
+		 			  GList               *file_list)
 {
 	GthEditCommentPage  *self;
 	GtkTextBuffer       *buffer;
@@ -64,15 +63,16 @@ gth_edit_comment_page_real_set_file (GthEditMetadataPage *base,
 	GthStringList       *tags;
 	GthMetadataProvider *provider;
 	gboolean             no_provider;
+	GthFileData         *file_data;
+	const char          *mime_type;
 
 	self = GTH_EDIT_COMMENT_PAGE (base);
 
 	_g_object_unref (self->priv->info);
-	self->priv->info = g_file_info_new ();
-	g_file_info_copy_into (file_data->info, self->priv->info);
+	self->priv->info = gth_file_data_list_get_common_info (file_list, "general::description,general::title,general::location,general::datetime,general::tags,general::rating");
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (GET_WIDGET ("note_text")));
-	metadata = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, "general::description");
+	metadata = (GthMetadata *) g_file_info_get_attribute_object (self->priv->info, "general::description");
 	if (metadata != NULL) {
 		GtkTextIter iter;
 
@@ -83,19 +83,19 @@ gth_edit_comment_page_real_set_file (GthEditMetadataPage *base,
 	else
 		gtk_text_buffer_set_text (buffer, "", -1);
 
-	metadata = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, "general::title");
+	metadata = (GthMetadata *) g_file_info_get_attribute_object (self->priv->info, "general::title");
 	if (metadata != NULL)
 		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("title_entry")), gth_metadata_get_formatted (metadata));
 	else
 		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("title_entry")), "");
 
-	metadata = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, "general::location");
+	metadata = (GthMetadata *) g_file_info_get_attribute_object (self->priv->info, "general::location");
 	if (metadata != NULL)
 		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("place_entry")), gth_metadata_get_formatted (metadata));
 	else
 		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("place_entry")), "");
 
-	metadata = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, "general::datetime");
+	metadata = (GthMetadata *) g_file_info_get_attribute_object (self->priv->info, "general::datetime");
 	if (metadata != NULL) {
 		gtk_combo_box_set_active (GTK_COMBO_BOX (self->priv->date_combobox), FOLLOWING_DATE);
 		gth_time_selector_set_exif_date (GTH_TIME_SELECTOR (self->priv->date_selector), gth_metadata_get_raw (metadata));
@@ -105,7 +105,7 @@ gth_edit_comment_page_real_set_file (GthEditMetadataPage *base,
 		gth_time_selector_set_exif_date (GTH_TIME_SELECTOR (self->priv->date_selector), "");
 	}
 
-	tags = (GthStringList *) g_file_info_get_attribute_object (file_data->info, "general::tags");
+	tags = (GthStringList *) g_file_info_get_attribute_object (self->priv->info, "general::tags");
 	if (tags != NULL) {
 		char *value;
 
@@ -117,7 +117,7 @@ gth_edit_comment_page_real_set_file (GthEditMetadataPage *base,
 	else
 		gth_tags_entry_set_text (GTH_TAGS_ENTRY (self->priv->tags_entry), "");
 
-	metadata = (GthMetadata *) g_file_info_get_attribute_object (file_data->info, "general::rating");
+	metadata = (GthMetadata *) g_file_info_get_attribute_object (self->priv->info, "general::rating");
 	if (metadata != NULL) {
 		int v;
 
@@ -131,32 +131,41 @@ gth_edit_comment_page_real_set_file (GthEditMetadataPage *base,
 
 	no_provider = TRUE;
 
-  	provider = gth_main_get_metadata_writer ("general::description", gth_file_data_get_mime_type (file_data));
+	if (file_list->next == NULL) {
+		GthFileData *first = file_list->data;
+		file_data = gth_file_data_new (first->file, first->info);
+	}
+	else
+		file_data = gth_file_data_new (NULL, ((GthFileData *) file_list->data)->info);
+
+	mime_type = gth_file_data_get_mime_type (file_data);
+
+  	provider = gth_main_get_metadata_writer ("general::description", mime_type);
 	gtk_widget_set_sensitive (GET_WIDGET ("note_text"), provider != NULL);
 	if (no_provider && (provider != NULL))
 		no_provider = FALSE;
 	_g_object_unref (provider);
 
-	provider = gth_main_get_metadata_writer ("general::location", gth_file_data_get_mime_type (file_data));
+	provider = gth_main_get_metadata_writer ("general::location", mime_type);
 	gtk_widget_set_sensitive (GET_WIDGET ("place_entry"), provider != NULL);
 	if (no_provider && (provider != NULL))
 		no_provider = FALSE;
 	_g_object_unref (provider);
 
-	provider = gth_main_get_metadata_writer ("general::datetime", gth_file_data_get_mime_type (file_data));
+	provider = gth_main_get_metadata_writer ("general::datetime", mime_type);
 	gtk_widget_set_sensitive (self->priv->date_combobox, provider != NULL);
 	gtk_widget_set_sensitive (self->priv->date_selector, provider != NULL);
 	if (no_provider && (provider != NULL))
 		no_provider = FALSE;
 	_g_object_unref (provider);
 
-	provider = gth_main_get_metadata_writer ("general::tags", gth_file_data_get_mime_type (file_data));
+	provider = gth_main_get_metadata_writer ("general::tags", mime_type);
 	gtk_widget_set_sensitive (self->priv->tags_entry, provider != NULL);
 	if (no_provider && (provider != NULL))
 		no_provider = FALSE;
 	_g_object_unref (provider);
 
-	provider = gth_main_get_metadata_writer ("general::rating", gth_file_data_get_mime_type (file_data));
+	provider = gth_main_get_metadata_writer ("general::rating", mime_type);
 	gtk_widget_set_sensitive (GET_WIDGET ("rating_spinbutton"), provider != NULL);
 	if (no_provider && (provider != NULL))
 		no_provider = FALSE;
@@ -166,14 +175,18 @@ gth_edit_comment_page_real_set_file (GthEditMetadataPage *base,
 		gtk_widget_hide (GTK_WIDGET (self));
 	else
 		gtk_widget_show (GTK_WIDGET (self));
+
+	g_object_unref (file_data);
 }
 
 
 void
 gth_edit_comment_page_real_update_info (GthEditMetadataPage *base,
-					GFileInfo           *info)
+					GFileInfo           *info,
+					gboolean             only_modified_fields)
 {
 	GthEditCommentPage  *self;
+	GthFileData         *file_data;
 	GtkTextBuffer       *buffer;
 	GtkTextIter          start;
 	GtkTextIter          end;
@@ -185,55 +198,66 @@ gth_edit_comment_page_real_update_info (GthEditMetadataPage *base,
 	GthStringList       *string_list;
 	GthDateTime         *date_time;
 	char                *exif_date;
+	char                *s;
 
 	self = GTH_EDIT_COMMENT_PAGE (base);
 
+	file_data = gth_file_data_new (NULL, self->priv->info);
+
 	/* caption */
 
-	metadata = g_object_new (GTH_TYPE_METADATA,
-				 "id", "general::title",
-				 "raw", gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("title_entry"))),
-				 "formatted", gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("title_entry"))),
-				 NULL);
-	g_file_info_set_attribute_object (info, "general::title", G_OBJECT (metadata));
-	g_object_unref (metadata);
+	if (! only_modified_fields || ! gth_file_data_attribute_equal (file_data, "general::title", gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("title_entry"))))) {
+		metadata = g_object_new (GTH_TYPE_METADATA,
+					 "id", "general::title",
+					 "raw", gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("title_entry"))),
+					 "formatted", gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("title_entry"))),
+					 NULL);
+		g_file_info_set_attribute_object (info, "general::title", G_OBJECT (metadata));
+		g_object_unref (metadata);
+	}
 
 	/* comment */
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (GET_WIDGET ("note_text")));
 	gtk_text_buffer_get_bounds (buffer, &start, &end);
 	text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-	metadata = g_object_new (GTH_TYPE_METADATA,
-				 "id", "general::description",
-				 "raw", text,
-				 "formatted", text,
-				 NULL);
-	g_file_info_set_attribute_object (info, "general::description", G_OBJECT (metadata));
-	g_object_unref (metadata);
+	if (! only_modified_fields || ! gth_file_data_attribute_equal (file_data, "general::description", text)) {
+		metadata = g_object_new (GTH_TYPE_METADATA,
+					 "id", "general::description",
+					 "raw", text,
+					 "formatted", text,
+					 NULL);
+		g_file_info_set_attribute_object (info, "general::description", G_OBJECT (metadata));
+		g_object_unref (metadata);
+	}
 	g_free (text);
 
 	/* location */
 
-	metadata = g_object_new (GTH_TYPE_METADATA,
-				 "id", "general::location",
-				 "raw", gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("place_entry"))),
-				 "formatted", gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("place_entry"))),
-				 NULL);
-	g_file_info_set_attribute_object (info, "general::location", G_OBJECT (metadata));
-	g_object_unref (metadata);
+	if (! only_modified_fields || ! gth_file_data_attribute_equal (file_data, "general::location", gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("place_entry"))))) {
+		metadata = g_object_new (GTH_TYPE_METADATA,
+					 "id", "general::location",
+					 "raw", gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("place_entry"))),
+					 "formatted", gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("place_entry"))),
+					 NULL);
+		g_file_info_set_attribute_object (info, "general::location", G_OBJECT (metadata));
+		g_object_unref (metadata);
+	}
 
 	/* date */
 
 	date_time = gth_datetime_new ();
 	gth_time_selector_get_value (GTH_TIME_SELECTOR (self->priv->date_selector), date_time);
 	exif_date = gth_datetime_to_exif_date (date_time);
-	metadata = g_object_new (GTH_TYPE_METADATA,
-				 "id", "general::datetime",
-				 "raw", exif_date,
-				 "formatted", exif_date,
-				 NULL);
-	g_file_info_set_attribute_object (info, "general::datetime", G_OBJECT (metadata));
-	g_object_unref (metadata);
+	if (! only_modified_fields || ! gth_file_data_attribute_equal (file_data, "general::datetime", exif_date)) {
+		metadata = g_object_new (GTH_TYPE_METADATA,
+					 "id", "general::datetime",
+					 "raw", exif_date,
+					 "formatted", exif_date,
+					 NULL);
+		g_file_info_set_attribute_object (info, "general::datetime", G_OBJECT (metadata));
+		g_object_unref (metadata);
+	}
 	gth_datetime_free (date_time);
 
 	/* tags */
@@ -243,31 +267,40 @@ gth_edit_comment_page_real_update_info (GthEditMetadataPage *base,
 	for (i = 0; tagv[i] != NULL; i++)
 		tags = g_list_prepend (tags, tagv[i]);
 	tags = g_list_reverse (tags);
-	string_list = gth_string_list_new (tags);
-	g_file_info_set_attribute_object (info, "general::tags", G_OBJECT (string_list));
+	if (tags != NULL)
+		string_list = gth_string_list_new (tags);
+	else
+		string_list = NULL;
+	if (! only_modified_fields || ! gth_file_data_attribute_equal_string_list (file_data, "general::tags", string_list)) {
+		if (string_list != NULL)
+			g_file_info_set_attribute_object (info, "general::tags", G_OBJECT (string_list));
+		else
+			g_file_info_remove_attribute (info, "general::tags");
+	}
 
 	/* rating */
 
-	if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (GET_WIDGET ("rating_spinbutton"))) > 0) {
-		char *s;
-
-		s = g_strdup_printf ("%d", gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (GET_WIDGET ("rating_spinbutton"))));
-		metadata = g_object_new (GTH_TYPE_METADATA,
-					 "id", "general::rating",
-					 "raw", s,
-					 "formatted", s,
-					 NULL);
-		g_file_info_set_attribute_object (info, "general::rating", G_OBJECT (metadata));
-		g_object_unref (metadata);
-		g_free (s);
+	s = g_strdup_printf ("%d", gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (GET_WIDGET ("rating_spinbutton"))));
+	if (! only_modified_fields || ! gth_file_data_attribute_equal (file_data, "general::rating", s)) {
+		if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (GET_WIDGET ("rating_spinbutton"))) > 0) {
+			metadata = g_object_new (GTH_TYPE_METADATA,
+						 "id", "general::rating",
+						 "raw", s,
+						 "formatted", s,
+						 NULL);
+			g_file_info_set_attribute_object (info, "general::rating", G_OBJECT (metadata));
+			g_object_unref (metadata);
+		}
+		else
+			g_file_info_remove_attribute (info, "general::rating");
 	}
-	else
-		g_file_info_remove_attribute (info, "general::rating");
 
+	g_free (s);
 	g_free (exif_date);
-	g_object_unref (string_list);
+	_g_object_unref (string_list);
 	g_strfreev (tagv);
 	g_list_free (tags);
+	g_object_unref (file_data);
 }
 
 
@@ -375,6 +408,24 @@ date_combobox_changed_cb (GtkComboBox *widget,
 }
 
 
+static void
+tags_entry_list_collapsed_cb (GthTagsEntry *widget,
+			      gpointer     user_data)
+{
+	GtkWidget *toplevel;
+	int        width;
+
+	/* collapse the dialog height */
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (widget));
+	if (! gtk_widget_is_toplevel (toplevel))
+		return;
+
+	gtk_window_get_size (GTK_WINDOW (toplevel), &width, NULL);
+	gtk_window_resize (GTK_WINDOW (toplevel), width, 1);
+}
+
+
 static void
 gth_edit_comment_page_init (GthEditCommentPage *self)
 {
@@ -384,42 +435,47 @@ gth_edit_comment_page_init (GthEditCommentPage *self)
 	gtk_container_set_border_width (GTK_CONTAINER (self), 12);
 
 	self->priv->builder = _gtk_builder_new_from_file ("edit-comment-page.ui", "edit_metadata");
-  	gtk_box_pack_start (GTK_BOX (self), _gtk_builder_get_widget (self->priv->builder, "content"), TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (self), _gtk_builder_get_widget (self->priv->builder, "content"), TRUE, TRUE, 0);
 
-  	self->priv->date_combobox = gtk_combo_box_new_text ();
-  	_gtk_combo_box_append_texts (GTK_COMBO_BOX (self->priv->date_combobox),
-  				     _("No date"),
-  				     _("The following date"),
-  				     _("Current date"),
-  				     _("Date photo was taken"),
-  				     _("Last modified date"),
-  				     _("File creation date"),
-  				     _("Do not modify"),
-  				     NULL);
-  	gtk_widget_show (self->priv->date_combobox);
-  	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("date_combobox_container")), self->priv->date_combobox, TRUE, TRUE, 0);
+	self->priv->date_combobox = gtk_combo_box_new_text ();
+	_gtk_combo_box_append_texts (GTK_COMBO_BOX (self->priv->date_combobox),
+				     _("No date"),
+				     _("The following date"),
+				     _("Current date"),
+				     _("Date photo was taken"),
+				     _("Last modified date"),
+				     _("File creation date"),
+				     _("Do not modify"),
+				     NULL);
+	gtk_widget_show (self->priv->date_combobox);
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("date_combobox_container")), self->priv->date_combobox, TRUE, TRUE, 0);
 
-  	g_signal_connect (self->priv->date_combobox,
+	g_signal_connect (self->priv->date_combobox,
 			  "changed",
 			  G_CALLBACK (date_combobox_changed_cb),
 			  self);
 
-  	self->priv->date_selector = gth_time_selector_new ();
-  	gtk_widget_show (self->priv->date_selector);
-  	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("date_selector_container")), self->priv->date_selector, FALSE, FALSE, 0);
+	self->priv->date_selector = gth_time_selector_new ();
+	gtk_widget_show (self->priv->date_selector);
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("date_selector_container")), self->priv->date_selector, FALSE, FALSE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("date_label")), self->priv->date_combobox);
 
-  	self->priv->tags_entry = gth_tags_entry_new ();
-  	gtk_widget_show (self->priv->tags_entry);
-  	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("tags_entry_container")), self->priv->tags_entry, FALSE, FALSE, 0);
+	self->priv->tags_entry = gth_tags_entry_new ();
+	gtk_widget_show (self->priv->tags_entry);
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("tags_entry_container")), self->priv->tags_entry, FALSE, FALSE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("tags_label")), self->priv->tags_entry);
+
+	g_signal_connect (self->priv->tags_entry,
+			  "list-collapsed",
+			  G_CALLBACK (tags_entry_list_collapsed_cb),
+			  self);
 }
 
 
 static void
 gth_edit_comment_page_gth_edit_comment_page_interface_init (GthEditMetadataPageIface *iface)
 {
-	iface->set_file = gth_edit_comment_page_real_set_file;
+	iface->set_file_list = gth_edit_comment_page_real_set_file_list;
 	iface->update_info = gth_edit_comment_page_real_update_info;
 	iface->get_name = gth_edit_comment_page_real_get_name;
 }
