@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -151,8 +150,6 @@ import_dialog_response_cb (GtkDialog *dialog,
 				GthSubfolderType     subfolder_type;
 				GthSubfolderFormat   subfolder_format;
 				char                *custom_format;
-				gboolean             overwrite_files;
-				gboolean             adjust_orientation;
 				GthTask             *task;
 
 				destination = gth_import_preferences_get_destination ();
@@ -160,8 +157,6 @@ import_dialog_response_cb (GtkDialog *dialog,
 				subfolder_format = eel_gconf_get_enum (PREF_IMPORT_SUBFOLDER_FORMAT, GTH_TYPE_SUBFOLDER_FORMAT, GTH_SUBFOLDER_FORMAT_YYYYMMDD);
 				single_subfolder = eel_gconf_get_boolean (PREF_IMPORT_SUBFOLDER_SINGLE, FALSE);
 				custom_format = eel_gconf_get_string (PREF_IMPORT_SUBFOLDER_CUSTOM_FORMAT, "");
-				overwrite_files = eel_gconf_get_boolean (PREF_IMPORT_OVERWRITE, FALSE);
-				adjust_orientation = eel_gconf_get_boolean (PREF_IMPORT_ADJUST_ORIENTATION, FALSE);
 
 				task = gth_import_task_new (data->browser,
 							    file_list,
@@ -173,8 +168,8 @@ import_dialog_response_cb (GtkDialog *dialog,
 							    (photoset->title != NULL ? photoset->title : ""),
 							    NULL,
 							    FALSE,
-							    overwrite_files,
-							    adjust_orientation);
+							    FALSE,
+							    FALSE);
 				gth_browser_exec_task (data->browser, task, FALSE);
 				gtk_widget_destroy (data->dialog);
 
@@ -366,9 +361,24 @@ list_photos_ready_cb (GObject      *source_object,
 	list = NULL;
 	for (scan = data->photos; scan; scan = scan->next) {
 		FlickrPhoto *photo = scan->data;
+		const char  *url;
 		GthFileData *file_data;
 
-		file_data = gth_file_data_new_for_uri (photo->url_o, (photo->mime_type != NULL) ? photo->mime_type : "image/jpeg");
+		url = NULL;
+		if (photo->url_s != NULL)
+			url = photo->url_s;
+		else if (photo->url_m != NULL)
+			url = photo->url_m;
+		else if (photo->url_o != NULL)
+			url = photo->url_o;
+
+		if (url == NULL)
+			url = photo->url_sq;
+
+		if (url == NULL)
+			continue;
+
+		file_data = gth_file_data_new_for_uri (url, (photo->mime_type != NULL) ? photo->mime_type : "image/jpeg");
 		g_file_info_set_file_type (file_data->info, G_FILE_TYPE_REGULAR);
 		g_file_info_set_size (file_data->info, FAKE_SIZE); /* set a fake size to make the progress dialog work correctly */
 		g_file_info_set_attribute_object (file_data->info, "flickr::object", G_OBJECT (photo));
@@ -401,6 +411,8 @@ photoset_combobox_changed_cb (GtkComboBox *widget,
 			    PHOTOSET_DATA_COLUMN, &data->photoset,
 			    -1);
 
+	gth_import_preferences_dialog_set_event (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog), data->photoset->title);
+
 	gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
 	flickr_service_list_photos (data->service,
 				    data->photoset,
@@ -500,16 +512,6 @@ file_list_selection_changed_cb (GtkIconView *iconview,
 }
 
 
-static void
-preferences_button_clicked_cb (GtkWidget  *widget,
-			       DialogData *data)
-{
-	gth_import_preferences_dialog_set_event (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog),
-						 (data->photoset != NULL) ? data->photoset->title : "");
-	gtk_window_present (GTK_WINDOW (data->preferences_dialog));
-}
-
-
 void
 dlg_import_from_flickr (FlickrServer *server,
 		        GthBrowser   *browser)
@@ -573,6 +575,13 @@ dlg_import_from_flickr (FlickrServer *server,
 	data->preferences_dialog = gth_import_preferences_dialog_new ();
 	gtk_window_set_transient_for (GTK_WINDOW (data->preferences_dialog), GTK_WINDOW (data->dialog));
 
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("destination_button_box")),
+			    gth_import_destination_button_new (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog)),
+			    TRUE,
+			    TRUE,
+			    0);
+	gtk_widget_show_all (GET_WIDGET ("destination_button_box"));
+
 	title = g_strdup_printf (_("Import from %s"), data->server->name);
 	gtk_window_set_title (GTK_WINDOW (data->dialog), title);
 	g_free (title);
@@ -605,12 +614,9 @@ dlg_import_from_flickr (FlickrServer *server,
 			  "selection_changed",
 			  G_CALLBACK (file_list_selection_changed_cb),
 			  data);
-	g_signal_connect (GET_WIDGET ("preferences_button"),
-			  "clicked",
-			  G_CALLBACK (preferences_button_clicked_cb),
-			  data);
 
 	update_selection_status (data);
+	gth_import_preferences_dialog_set_event (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog), "");
 
 	data->conn = flickr_connection_new (data->server);
 	data->service = flickr_service_new (data->conn);
