@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -31,6 +30,7 @@
 #include "preferences.h"
 
 
+#define DEFAULT_FONT_NAME "sans normal 10"
 #define GET_WIDGET(name) _gtk_builder_get_widget (self->priv->builder, (name))
 
 
@@ -135,7 +135,7 @@ gth_image_print_job_init (GthImagePrintJob *self)
 	self->priv->page_setup = NULL;
 	self->priv->current_page = 0;
 	self->priv->caption_attributes = eel_gconf_get_string (PREF_IMAGE_PRINT_CAPTION, "");
-	self->priv->font_name = eel_gconf_get_string (PREF_IMAGE_PRINT_FONT_NAME, "sans 10");
+	self->priv->font_name = eel_gconf_get_string (PREF_IMAGE_PRINT_FONT_NAME, DEFAULT_FONT_NAME);
 	self->priv->selected = NULL;
 	self->priv->requested_images_per_page = eel_gconf_get_integer (PREF_IMAGE_PRINT_IMAGES_PER_PAGE, 1);
 	self->priv->unit = eel_gconf_get_enum (PREF_IMAGE_PRINT_UNIT, GTH_TYPE_METRIC, GTH_METRIC_PIXELS);
@@ -852,17 +852,19 @@ preview_expose_event_cb (GtkWidget      *widget,
 {
 	GthImagePrintJob *self = user_data;
 	cairo_t          *cr;
+	GtkAllocation     allocation;
 	PangoLayout      *pango_layout;
 
 	g_return_if_fail (GTH_IS_IMAGE_PRINT_JOB (self));
 	g_return_if_fail ((self->priv->page_setup != NULL) && GTK_IS_PAGE_SETUP (self->priv->page_setup));
 
-	cr = gdk_cairo_create (widget->window);
+	cr = gdk_cairo_create (gtk_widget_get_window (widget));
 
 	/* paint the paper */
 
 	cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
-	cairo_rectangle (cr, 0, 0, widget->allocation.width - 1, widget->allocation.height - 1);
+	gtk_widget_get_allocation (widget, &allocation);
+	cairo_rectangle (cr, 0, 0, allocation.width - 1, allocation.height - 1);
 	cairo_fill_preserve (cr);
 	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
 	cairo_stroke (cr);
@@ -1504,7 +1506,8 @@ print_operation_done_cb (GtkPrintOperation       *operation,
 
 
 GthImagePrintJob *
-gth_image_print_job_new (GList *file_data_list)
+gth_image_print_job_new (GList   *file_data_list,
+			 GError **error)
 {
 	GthImagePrintJob *self;
 	GList            *scan;
@@ -1514,12 +1517,24 @@ gth_image_print_job_new (GList *file_data_list)
 
 	self->priv->n_images = g_list_length (file_data_list);
 	self->priv->images = g_new (GthImageInfo *, self->priv->n_images + 1);
-	for (scan = file_data_list, n = 0; scan; scan = scan->next)
-		self->priv->images[n++] = gth_image_info_new ((GthFileData *) scan->data);
+	for (scan = file_data_list, n = 0; scan; scan = scan->next) {
+		GthFileData *file_data = scan->data;
+
+		if (_g_mime_type_is_image (gth_file_data_get_mime_type (file_data)))
+			self->priv->images[n++] = gth_image_info_new (file_data);
+	}
 	self->priv->images[n] = NULL;
+	self->priv->n_images = n;
 	self->priv->image_width = 0;
 	self->priv->image_height = 0;
 
+	if (self->priv->n_images == 0) {
+		if (error != NULL)
+			*error = g_error_new_literal (GTH_ERROR, GTH_ERROR_GENERIC, _("No valid file selected."));
+		g_object_unref (self);
+		return NULL;
+	}
+
 	self->priv->print_operation = gtk_print_operation_new ();
 	gtk_print_operation_set_allow_async (self->priv->print_operation, TRUE);
 	gtk_print_operation_set_custom_tab_label (self->priv->print_operation, _("Layout"));
