@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -25,10 +24,10 @@
 #include <gdk/gdkkeysyms.h>
 #include <gthumb.h>
 #include "gth-image-viewer-page.h"
-
+#include "preferences.h"
 
 #define GTH_IMAGE_VIEWER_PAGE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTH_TYPE_IMAGE_VIEWER_PAGE, GthImageViewerPagePrivate))
-#define GCONF_NOTIFICATIONS 7
+#define GCONF_NOTIFICATIONS 8
 
 
 struct _GthImageViewerPagePrivate {
@@ -45,6 +44,7 @@ struct _GthImageViewerPagePrivate {
 	guint              hide_mouse_timeout;
 	guint              motion_signal;
 	gboolean           pixbuf_changed;
+	gboolean           shrink_wrap;
 };
 
 static gpointer gth_image_viewer_page_parent_class = NULL;
@@ -151,7 +151,7 @@ static void
 image_viewer_activate_action_view_shrink_wrap (GtkAction          *action,
 					       GthImageViewerPage *self)
 {
-	gth_image_viewer_page_shrink_wrap (self);
+	eel_gconf_set_boolean (PREF_VIEWER_SHRINK_WRAP, gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
 
 
@@ -190,11 +190,15 @@ static GtkActionEntry image_viewer_action_entries[] = {
 	  N_("Width"), "",
 	  N_("Zoom to fit width"),
 	  G_CALLBACK (image_viewer_activate_action_view_zoom_fit_width) },
+};
 
+
+static GtkToggleActionEntry image_viewer_toggle_action_entries[] = {
 	{ "ImageViewer_View_ShrinkWrap", NULL,
 	  N_("_Fit Window to Image"), "<control>e",
 	  N_("Resize the window to the size of the image"),
-	  G_CALLBACK (image_viewer_activate_action_view_shrink_wrap) },
+	  G_CALLBACK (image_viewer_activate_action_view_shrink_wrap),
+	  FALSE }
 };
 
 
@@ -202,6 +206,9 @@ static void
 viewer_image_ready_cb (GtkWidget          *widget,
 		       GthImageViewerPage *self)
 {
+	if (self->priv->shrink_wrap)
+		gth_image_viewer_page_shrink_wrap (self, TRUE);
+
 	gth_image_history_clear (self->priv->history);
 	gth_image_history_add_image (self->priv->history,
 				     gth_image_viewer_get_current_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer)),
@@ -395,11 +402,24 @@ pref_reset_scrollbars_changed (GConfClient *client,
 }
 
 
+static void
+pref_viewer_shrink_wrap_changed (GConfClient *client,
+				 guint        cnxn_id,
+				 GConfEntry  *entry,
+				 gpointer     user_data)
+{
+	GthImageViewerPage *self = user_data;
+
+	gth_image_viewer_page_shrink_wrap (self, eel_gconf_get_boolean (PREF_VIEWER_SHRINK_WRAP, FALSE));
+}
+
+
 static void
 gth_image_viewer_page_real_activate (GthViewerPage *base,
 				     GthBrowser    *browser)
 {
 	GthImageViewerPage *self;
+	GtkAction          *action;
 	int                 i;
 
 	self = (GthImageViewerPage*) base;
@@ -412,6 +432,10 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 				      image_viewer_action_entries,
 				      G_N_ELEMENTS (image_viewer_action_entries),
 				      self);
+	gtk_action_group_add_toggle_actions (self->priv->actions,
+					     image_viewer_toggle_action_entries,
+					     G_N_ELEMENTS (image_viewer_toggle_action_entries),
+					     self);
 	gtk_ui_manager_insert_action_group (gth_browser_get_ui_manager (browser), self->priv->actions, 0);
 
 	self->priv->preloader = gth_browser_get_image_preloader (browser);
@@ -429,6 +453,11 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 	gth_image_viewer_set_black_background (GTH_IMAGE_VIEWER (self->priv->viewer), eel_gconf_get_boolean (PREF_BLACK_BACKGROUND, FALSE));
 	gth_image_viewer_set_reset_scrollbars (GTH_IMAGE_VIEWER (self->priv->viewer), eel_gconf_get_boolean (PREF_RESET_SCROLLBARS, TRUE));
 
+	self->priv->shrink_wrap = eel_gconf_get_boolean (PREF_VIEWER_SHRINK_WRAP, FALSE);
+	action = gtk_action_group_get_action (self->priv->actions, "ImageViewer_View_ShrinkWrap");
+	if (action != NULL)
+		g_object_set (action, "active", self->priv->shrink_wrap, NULL);
+
 	gtk_widget_show (self->priv->viewer);
 
 	g_signal_connect (G_OBJECT (self->priv->viewer),
@@ -506,6 +535,11 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 					   PREF_RESET_SCROLLBARS,
 					   pref_reset_scrollbars_changed,
 					   self);
+
+	self->priv->cnxn_id[i++] = eel_gconf_notification_add (
+					   PREF_VIEWER_SHRINK_WRAP,
+					   pref_viewer_shrink_wrap_changed,
+					   self);
 }
 
 
@@ -941,7 +975,7 @@ _gth_image_viewer_page_set_pixbuf (GthImageViewerPage *self,
 	g_file_info_set_attribute_int32 (file_data->info, "image::width", width);
 	g_file_info_set_attribute_int32 (file_data->info, "image::height", height);
 
-	size = g_strdup_printf ("%d × %d", width, height);
+	size = g_strdup_printf (_("%d × %d"), width, height);
 	g_file_info_set_attribute_string (file_data->info, "general::dimensions", size);
 
 	gth_monitor_metadata_changed (gth_main_get_default_monitor (), file_data);
@@ -1013,6 +1047,7 @@ gth_image_viewer_page_instance_init (GthImageViewerPage *self)
 {
 	self->priv = GTH_IMAGE_VIEWER_PAGE_GET_PRIVATE (self);
 	self->priv->history = gth_image_history_new ();
+	self->priv->shrink_wrap = FALSE;
 }
 
 
@@ -1112,6 +1147,23 @@ gth_image_viewer_page_reset (GthImageViewerPage *self)
 }
 
 
+static int
+add_non_content_width (GthImageViewerPage *self,
+		       GtkWidget          *non_content)
+{
+	int width = 0;
+
+	if ((non_content != NULL) && gtk_widget_get_visible (non_content)) {
+		GtkAllocation allocation;
+
+		gtk_widget_get_allocation (non_content, &allocation);
+		width = allocation.width;
+	}
+
+	return width;
+}
+
+
 static int
 add_non_content_height (GthImageViewerPage *self,
 			GtkWidget          *non_content)
@@ -1120,6 +1172,7 @@ add_non_content_height (GthImageViewerPage *self,
 
 	if ((non_content != NULL) && gtk_widget_get_visible (non_content)) {
 		GtkAllocation allocation;
+
 		gtk_widget_get_allocation (non_content, &allocation);
 		height = allocation.height;
 	}
@@ -1129,7 +1182,8 @@ add_non_content_height (GthImageViewerPage *self,
 
 
 void
-gth_image_viewer_page_shrink_wrap (GthImageViewerPage *self)
+gth_image_viewer_page_shrink_wrap (GthImageViewerPage *self,
+				   gboolean            activate)
 {
 	GdkPixbuf *pixbuf;
 	int        width;
@@ -1141,6 +1195,26 @@ gth_image_viewer_page_shrink_wrap (GthImageViewerPage *self)
 	int        max_width;
 	int        max_height;
 
+	self->priv->shrink_wrap = activate;
+	if (! self->priv->shrink_wrap) {
+		int width;
+		int height;
+
+		if (gth_window_get_page_size (GTH_WINDOW (self->priv->browser),
+					      GTH_BROWSER_PAGE_BROWSER,
+					      &width,
+					      &height))
+		{
+			gth_window_save_page_size (GTH_WINDOW (self->priv->browser), GTH_BROWSER_PAGE_VIEWER, width, height);
+			gth_window_apply_saved_size (GTH_WINDOW (self->priv->browser), GTH_BROWSER_PAGE_VIEWER);
+		}
+		else
+			gth_window_clear_saved_size (GTH_WINDOW (self->priv->browser), GTH_BROWSER_PAGE_VIEWER);
+		gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_FIT_SIZE_IF_LARGER);
+
+		return;
+	}
+
 	pixbuf = gth_image_viewer_page_get_pixbuf (self);
 	if (pixbuf == NULL)
 		return;
@@ -1155,23 +1229,33 @@ gth_image_viewer_page_shrink_wrap (GthImageViewerPage *self)
 	other_height += add_non_content_height (self, gth_window_get_area (GTH_WINDOW (self->priv->browser), GTH_WINDOW_TOOLBAR));
 	other_height += add_non_content_height (self, gth_window_get_area (GTH_WINDOW (self->priv->browser), GTH_WINDOW_STATUSBAR));
 	other_height += add_non_content_height (self, gth_browser_get_viewer_toolbar (self->priv->browser));
+	if (eel_gconf_get_enum (PREF_UI_VIEWER_THUMBNAILS_ORIENT, GTK_TYPE_ORIENTATION, GTK_ORIENTATION_HORIZONTAL) == GTK_ORIENTATION_HORIZONTAL)
+		other_height += add_non_content_height (self, gth_browser_get_thumbnail_list (self->priv->browser));
+	else
+		other_width += add_non_content_width (self, gth_browser_get_thumbnail_list (self->priv->browser));
+	other_width += add_non_content_width (self, gth_browser_get_viewer_sidebar (self->priv->browser));
 	other_width += 2;
 	other_height += 2;
 
 	screen = gtk_widget_get_screen (GTK_WIDGET (self->priv->browser));
-	max_width = gdk_screen_get_width (screen) * 9 / 10;
-	max_height = gdk_screen_get_height (screen) * 8 / 10;
+	max_width = round ((double) gdk_screen_get_width (screen) * 8.5 / 10.0);
+	max_height = round ((double) gdk_screen_get_height (screen) * 8.5 / 10.0);
 
 	if (width + other_width > max_width) {
-		width = max_width;
+		width = max_width - other_width;
 		height = width / ratio;
 	}
 
 	if (height + other_height > max_height) {
-		height = max_height;
+		height = max_height - other_height;
 		width = height * ratio;
 	}
 
-	gtk_window_resize (GTK_WINDOW (self->priv->browser), width + other_width, height + other_height);
-	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_FIT_SIZE);
+	gth_window_save_page_size (GTH_WINDOW (self->priv->browser),
+				   GTH_BROWSER_PAGE_VIEWER,
+				   width + other_width,
+				   height + other_height);
+	if (gth_window_get_current_page (GTH_WINDOW (self->priv->browser)) == GTH_BROWSER_PAGE_VIEWER)
+		gth_window_apply_saved_size (GTH_WINDOW (self->priv->browser), GTH_BROWSER_PAGE_VIEWER);
+	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_FIT_SIZE_IF_LARGER);
 }
