@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program. If not, see <http://www.gnu.org/licenses/>
  */
 
 #include <config.h>
@@ -79,8 +78,6 @@ destroy_dialog (gpointer user_data)
 		GthSubfolderType    subfolder_type;
 		GthSubfolderFormat  subfolder_format;
 		char               *custom_format;
-		gboolean            overwrite_files;
-		gboolean            adjust_orientation;
 		GtkWidget          *file_view;
 		GList              *items;
 		GList              *file_list;
@@ -90,8 +87,6 @@ destroy_dialog (gpointer user_data)
 		subfolder_type = eel_gconf_get_enum (PREF_IMPORT_SUBFOLDER_TYPE, GTH_TYPE_SUBFOLDER_TYPE, GTH_SUBFOLDER_TYPE_FILE_DATE);
 		subfolder_format = eel_gconf_get_enum (PREF_IMPORT_SUBFOLDER_FORMAT, GTH_TYPE_SUBFOLDER_FORMAT, GTH_SUBFOLDER_FORMAT_YYYYMMDD);
 		custom_format = eel_gconf_get_string (PREF_IMPORT_SUBFOLDER_CUSTOM_FORMAT, "");
-		overwrite_files = eel_gconf_get_boolean (PREF_IMPORT_OVERWRITE, FALSE);
-		adjust_orientation = eel_gconf_get_boolean (PREF_IMPORT_ADJUST_ORIENTATION, FALSE);
 
 		file_view = gth_file_list_get_view (GTH_FILE_LIST (data->file_list));
 		items = gth_file_selection_get_selected (GTH_FILE_SELECTION (file_view));
@@ -115,8 +110,8 @@ destroy_dialog (gpointer user_data)
 						    gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("event_entry"))),
 						    tags,
 						    delete_imported,
-						    overwrite_files,
-						    adjust_orientation);
+						    FALSE,
+						    eel_gconf_get_boolean (PREF_PHOTO_IMPORT_ADJUST_ORIENTATION, FALSE));
 			gth_browser_exec_task (data->browser, task, FALSE);
 
 			g_strfreev (tags);
@@ -201,6 +196,14 @@ ok_clicked_cb (GtkWidget  *widget,
 }
 
 
+static void
+help_clicked_cb (GtkWidget  *widget,
+                 DialogData *data)
+{
+        show_help_dialog (GTK_WINDOW (data->dialog), "gthumb-importing");
+}
+
+
 static void
 update_sensitivity (DialogData *data)
 {
@@ -465,12 +468,11 @@ filter_combobox_changed_cb (GtkComboBox *widget,
 
 
 static void
-preferences_button_clicked_cb (GtkWidget  *widget,
-			       DialogData *data)
+event_entry_changed_cb (GtkEditable *editable,
+			DialogData  *data)
 {
 	gth_import_preferences_dialog_set_event (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog),
 						 gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("event_entry"))));
-	gtk_window_present (GTK_WINDOW (data->preferences_dialog));
 }
 
 
@@ -585,6 +587,13 @@ dlg_photo_importer (GthBrowser *browser,
 	data->preferences_dialog = gth_import_preferences_dialog_new ();
 	gtk_window_set_transient_for (GTK_WINDOW (data->preferences_dialog), GTK_WINDOW (data->dialog));
 
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("destination_button_box")),
+			    gth_import_destination_button_new (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog)),
+			    TRUE,
+			    TRUE,
+			    0);
+	gtk_widget_show_all (GET_WIDGET ("destination_button_box"));
+
 	/* Set the signals handlers. */
 
 	g_signal_connect (G_OBJECT (data->dialog),
@@ -599,6 +608,10 @@ dlg_photo_importer (GthBrowser *browser,
 			  "clicked",
 			  G_CALLBACK (close_dialog),
 			  data);
+        g_signal_connect (GET_WIDGET ("help_button"),
+                          "clicked",
+                          G_CALLBACK (help_clicked_cb),
+                          data);
 	g_signal_connect (data->source_list,
 			  "changed",
 			  G_CALLBACK (source_list_changed_cb),
@@ -615,14 +628,14 @@ dlg_photo_importer (GthBrowser *browser,
 			  "selection_changed",
 			  G_CALLBACK (file_view_selection_changed_cb),
 			  data);
-	g_signal_connect (GET_WIDGET ("preferences_button"),
-			  "clicked",
-			  G_CALLBACK (preferences_button_clicked_cb),
-			  data);
 	data->entry_points_changed_id = g_signal_connect (gth_main_get_default_monitor (),
 							  "entry-points-changed",
 							  G_CALLBACK (entry_points_changed_cb),
 							  data);
+	g_signal_connect_after (GET_WIDGET ("event_entry"),
+				"changed",
+				G_CALLBACK (event_entry_changed_cb),
+				data);
 
 	/* Run dialog. */
 
@@ -630,5 +643,8 @@ dlg_photo_importer (GthBrowser *browser,
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	gtk_widget_show (data->dialog);
 
+	gth_import_preferences_dialog_set_event (GTH_IMPORT_PREFERENCES_DIALOG (data->preferences_dialog),
+						 gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("event_entry"))));
+
 	update_source_list (data);
 }
