@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -103,8 +102,7 @@ resize_step (GthPixbufTask *pixbuf_task)
 	}
 
 	if ((new_w > 1) && (new_h > 1)) {
-		if (pixbuf_task->dest != NULL)
-			g_object_unref (pixbuf_task->dest);
+		_g_object_unref (pixbuf_task->dest);
 		pixbuf_task->dest = _gdk_pixbuf_scale_simple_safe (pixbuf_task->src, new_w, new_h, GDK_INTERP_BILINEAR);
 	}
 	else
