@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef GTH_FILE_DATA_H
@@ -26,6 +25,7 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <gio/gio.h>
+#include "gth-string-list.h"
 
 G_BEGIN_DECLS
 
@@ -103,6 +103,15 @@ void          gth_file_data_ready_with_error        (GthFileData    *file_data,
 						     GError         *error);
 char *        gth_file_data_get_attribute_as_string (GthFileData    *file_data,
 				                     const char     *id);
+GFileInfo *   gth_file_data_list_get_common_info    (GList          *file_data_list,
+						     const char     *attribtues);
+gboolean      gth_file_data_attribute_equal         (GthFileData    *file_data,
+						     const char     *attribute,
+						     const char     *value);
+gboolean      gth_file_data_attribute_equal_string_list
+						    (GthFileData    *file_data,
+						     const char     *attribute,
+						     GthStringList  *value);
 
 G_END_DECLS
 
