@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -564,6 +563,9 @@ gth_main_get_metadata_category (const char *id)
 {
 	int i;
 
+	if (id == NULL)
+		return NULL;
+
 	for (i = 0; i < Main->priv->metadata_category->len; i++) {
 		GthMetadataCategory *category;
 
@@ -581,6 +583,9 @@ gth_main_get_metadata_info (const char *id)
 {
 	int i;
 
+	if (id == NULL)
+		return NULL;
+
 	for (i = 0; i < Main->priv->metadata_info->len; i++) {
 		GthMetadataInfo *info;
 
@@ -1221,6 +1226,7 @@ gth_main_activate_extensions (void)
 						"resize_images",
 						"search",
 						"slideshow",
+						"webalbums",
 						NULL };
 	int         i;
 	GSList     *active_extensions;
@@ -1319,8 +1325,11 @@ attribute_list_reload_required (const char *old_attributes,
 		g_object_ref (provider);
 	}
 
-	/*g_print ("attributes to load: %s\n", new_attributes);
-	g_print ("attributes not available: \n");*/
+	/*
+	g_print ("old attributes: %s\n", old_attributes);
+	g_print ("new attributes: %s\n", new_attributes);
+	g_print ("attributes not available: \n");
+	*/
 
 	reload_required = FALSE;
 	for (i = 0; ! reload_required && (i < new_attributes_len); i++)
@@ -1329,9 +1338,11 @@ attribute_list_reload_required (const char *old_attributes,
 		    && (strcmp (new_attributes_v[i], "none") != 0))
 		{
 			reload_required = TRUE;
-			/*g_print ("\t%s\n", new_attributes_v[i]);*/
+			/* g_print ("\t%s\n", new_attributes_v[i]); */
 		}
 
+	/* g_print ("reload required: %d\n", reload_required); */
+
 	g_strfreev (new_attributes_v);
 	g_strfreev (old_attributes_v);
 
