@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
   
 #ifndef GTH_TIME_H
@@ -64,7 +63,7 @@ void          gth_datetime_from_struct_tm  (GthDateTime *dt,
 void          gth_datetime_from_gdate      (GthDateTime *dt,
 			 	 	    GDate       *date);
 char *        gth_datetime_to_exif_date    (GthDateTime *dt);
-void          gth_datetime_to_struct_tm    (GthDateTime *dt,
+gboolean      gth_datetime_to_struct_tm    (GthDateTime *dt,
 					    struct tm   *tm);
 gboolean      gth_datetime_to_timeval      (GthDateTime *dt,
 					    GTimeVal    *tv);
